# -*- coding: utf-8 -*-
##############################################################################
#
#    Remembership module for OpenERP, Overload membership module
#    Copyright (C) 2012 L'Heureux Cyclage (<http://www.heureux-cyclage.org>) Ludovic CHEVALIER
#
#    This file is a part of Remembership
#
#    Remembership is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Remembership is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
import time


class Partner(osv.osv):
    _inherit = 'res.partner'

    _columns = {
        'member_ident': fields.char('Member identifier', size=64),
        'associate_members': fields.one2many('res.partner', 'associate_member', 'Associate members', help='Members who are associated to this partner.'),
    }

    _sql_constraints = [
        ('member_ident_uniq', 'unique(member_ident, company_id)', 'The member identifier must be unique !'),
    ]

Partner()


class Product(osv.osv):
    _inherit = 'product.product'

    _columns = {
        'membership_grouped': fields.boolean('Grouped membership product', help='Check if it\'s a grouped membership product.'),
        'membership_date2date': fields.boolean('Date to date membership product', required=False, help='Check if it\'s a date to date membership product.'),
    }

Product()


class account_invoice_line(osv.osv):
    _inherit = 'account.invoice.line'


    def write(self, cr, uid, ids, vals, context=None):
        member_line_obj = self.pool.get('membership.membership_line')
        res = super(account_invoice_line, self).write(cr, uid, ids, vals, context=context)
        for line in self.browse(cr, uid, ids, context=context):
            if line.invoice_id.type == 'out_invoice':
                ml_ids = member_line_obj.search(cr, uid, [('account_invoice_line', '=', line.id)], context=context)
                if line.product_id and line.product_id.membership:
                    date_from = line.product_id.membership_date_from
                    date_to = line.product_id.membership_date_to
                    if line.invoice_id.date_invoice > date_from and line.invoice_id.date_invoice < date_to:
                        date_from = line.invoice_id.date_invoice
                    if line.product_id.membership_grouped:
                        if line.invoice_id.partner_id.associate_members:
                            for associate_member in line.invoice_id.partner_id.associate_members:
                                member_line_obj.create(cr, uid, {
                                    'partner': associate_member.id,
                                    'membership_id': line.product_id.id,
                                    'member_price': line.price_unit,
                                    'date': time.strftime('%Y-%m-%d'),
                                    'date_from': date_from,
                                    'date_to': date_to,
                                    'account_invoice_line': line.id,
                                }, context=context)
                        else:
                            print("DEV: mettre une contrainte pour l'objet\
                                  membership.membership_line interdisant les\
                                  adhésions groupées reliées à des partenaires\
                                  sans membres associés")
                    else:
                        associate_ml_ids = member_line_obj.search(cr, uid, [('account_invoice_line', '=', line.id), ('partner', '!=', line.invoice_id.partner_id.id)], context=context)
                        member_line_obj.unlink(cr, uid, associate_ml_ids, context=context)

                    #Define member ident if it's necessary
                    partners = [line.invoice_id.partner_id]
                    if line.invoice_id.partner_id.associate_members:
                        partners.extend(line.invoice_id.partner_id.associate_members)
                    for i in partners:
                        if not i.member_ident:
                            mbr_id = self.pool.get('ir.sequence').get(cr, uid, 'member_ident')
                            self.pool.get('res.partner').write(cr, uid, i.id, {'member_ident': mbr_id})
                if line.product_id and not line.product_id.membership and ml_ids:
                    # Product line has changed to a non membership product
                    member_line_obj.unlink(cr, uid, ml_ids, context=context)
        return res

    def create(self, cr, uid, vals, context=None):
        member_line_obj = self.pool.get('membership.membership_line')
        res = super(account_invoice_line, self).create(cr, uid, vals, context=context)
        line = self.browse(cr, uid, res, context=context)

        if line.invoice_id.type == 'out_invoice' and line.product_id and line.product_id.membership:
            date_from = line.product_id.membership_date_from
            date_to = line.product_id.membership_date_to
            if line.invoice_id.date_invoice > date_from and line.invoice_id.date_invoice < date_to:
                date_from = line.invoice_id.date_invoice
            partners = [line.invoice_id.partner_id]
            if line.product_id.membership_grouped and line.invoice_id.partner_id.associate_members:
                partners.extend(line.invoice_id.partner_id.associate_members)
                #Adding membership lines just for associate partners
                for associate_member in line.invoice_id.partner_id.associate_members:
                    member_line_obj.create(cr, uid, {
                        'partner': associate_member.id,
                        'membership_id': line.product_id.id,
                        'member_price': line.price_unit,
                        'date': time.strftime('%Y-%m-%d'),
                        'date_from': date_from,
                        'date_to': date_to,
                        'account_invoice_line': line.id,
                    }, context=context)
            #Define member ident if it's necessary
            for i in partners:
                if not i.member_ident:
                    mbr_id = self.pool.get('ir.sequence').get(cr, uid, 'member_ident')
                    self.pool.get('res.partner').write(cr, uid, i.id, {'member_ident': mbr_id})
        return res

account_invoice_line()


class pos_make_payment(osv.osv_memory):
    _inherit = 'pos.make.payment'

    def check(self, cr, uid, ids, context=None):
        """
        Auto invoice orders. Temporarly solution before make partner members just with orders, without invoice.
        """
        res = super(pos_make_payment, self).check(cr, uid, ids, context=context)

        order_obj = self.pool.get('pos.order')
        order_obj.action_invoice(cr, uid, ids, context=context)

        return res

pos_make_payment()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
